@@ -1,4 +1,6 @@
 #
+# SPDX-License-Identifier: MIT
+#
 # Copyright (c) 2021-2022 Carsten Igel.
 #
 # This file is part of pdm-bump
@@ -8,11 +10,14 @@
 # Refer to LICENSE for more information
 #
 import sys
-from enum import IntEnum, auto
-from typing import Any, Optional, Protocol, Union, cast
+from enum import Enum, IntEnum, auto
+from functools import cached_property
+from io import BytesIO
+from pathlib import Path
+from typing import Any, Final, Iterable, Optional, Protocol, cast
 
-from pdm.project.project_file import PyProject
-from tomlkit.items import Table
+from pyproject_metadata import StandardMetadata
+from tomli_w import dump as dump_toml
 
 from .logging import logger, traced_function
 
@@ -22,16 +27,25 @@ if sys.version_info >= (3, 10, 0):
 else:
     from typing_extensions import TypeAlias
 
+if sys.version_info >= (3, 11, 0):
+    # suspicious mypy behavior
+    from tomllib import load as load_toml  # type: ignore
+else:
+    # Python 3.11 -> mypy will not recognize this
+    from tomli import load as load_toml  # type: ignore
+
 
 _ConfigMapping: TypeAlias = dict[str, Any]
-_ModernPyProject: TypeAlias = Union[_ConfigMapping, PyProject]
 
 
-class ConfigHolder(Protocol):
-    @property
-    def pyproject(self) -> _ModernPyProject:
-        # Method empty: Only a protocol stub
-        pass
+class _StringEnum(str, Enum):
+    pass
+
+
+# Justification: Minimal protocol
+class ConfigHolder(Protocol):  # pylint: disable=R0903
+    root: Path
+    PYPROJECT_FILENAME: str
 
     @property
     def config(self) -> _ConfigMapping:
@@ -40,7 +54,9 @@ class ConfigHolder(Protocol):
 
 
 @traced_function
-def _get_config_value(config: _ConfigMapping, *keys: str) -> Optional[Any]:
+def _get_config_value(
+    config: _ConfigMapping, *keys: str, default_value: Optional[Any] = None
+) -> Optional[Any]:
     front: str
 
     key: str = ".".join(keys)
@@ -55,11 +71,11 @@ def _get_config_value(config: _ConfigMapping, *keys: str) -> Optional[Any]:
             keys = tuple(keys[1:])
         else:
             logger.debug("Could not find configuration section %s.", front)
-            return None
+            return default_value
 
     front = keys[0]
 
-    result = None if front not in config.keys() else config[front]
+    result = default_value if front not in config.keys() else config[front]
     logger.debug("Found value at '%s' is: %s", key, result)
     return result
 
@@ -85,7 +101,37 @@ def _set_config_value(config: _ConfigMapping, value: Any, *keys: str) -> None:
     config[front] = value
 
 
+class ConfigSections(_StringEnum):
+    PDM_BUMP: Final[str] = "pdm_bump"
+    PDM_BUMP_VCS: Final[str] = "vcs"
+
+
+class ConfigKeys(_StringEnum):
+    VERSION: Final[str] = "version"
+    VERSION_SOURCE: Final[str] = "source"
+    VERSION_SOURCE_FILE_PATH: Final[str] = "path"
+    BUILD_BACKEND: Final[str] = "build-backend"
+    VCS_PROVIDER: Final[str] = "provider"
+    PROJECT_METADATA: Final[str] = "project"
+
+
+class ConfigValues(_StringEnum):
+    VERSION_SOURCE_FILE: Final[str] = "file"
+    BUILD_BACKEND_PDM_PEP517_API: Final[str] = "pdm.pep517.api"
+
+
+# Justification: Currently no more meta data to check
+class ProjectMetaData:  # pylint: disable=R0903
+    def __init__(self, meta_data: StandardMetadata) -> None:
+        self.__meta_data = meta_data
+
+    @property
+    def is_dynamic_version(self) -> bool:
+        return ConfigKeys().VERSION in self.__meta_data.dynamic
+
+
 class _ConfigSection(IntEnum):
+    ROOT = auto()
     METADATA = auto()
     PLUGIN_CONFIG = auto()
     BUILD_SYSTEM = auto()
@@ -96,6 +142,18 @@ class Config:
     def __init__(self, project: ConfigHolder) -> None:
         self.__project: ConfigHolder = project
 
+    @cached_property
+    @traced_function
+    def pyproject_file(self) -> Path:
+        return self.__project.root / self.__project.PYPROJECT_FILENAME
+
+    @property
+    @traced_function
+    def meta_data(self) -> ProjectMetaData:
+        data: _ConfigMapping = self._get_pyproject_config(_ConfigSection.ROOT)
+        meta: StandardMetadata = StandardMetadata.from_pyproject(data)
+        return ProjectMetaData(meta)
+
     @traced_function
     def get_pyproject_metadata(self, *keys: str) -> Optional[Any]:
         config: _ConfigMapping = self._get_pyproject_config(
@@ -136,66 +194,50 @@ class Config:
     @traced_function
     def set_pyproject_metadata(self, value: Any, *keys: str) -> None:
         config: _ConfigMapping = self._get_pyproject_config(
-            _ConfigSection.METADATA
+            _ConfigSection.ROOT
         )
-        _set_config_value(config, value, *keys)
+        new_config: _ConfigMapping = _get_config_value(
+            config, ConfigKeys.PROJECT_METADATA, default_value={}
+        )
+        _set_config_value(new_config, value, *keys)
+        self._write_config(config)
+
+    def _write_config(self, config: _ConfigMapping) -> None:
+        with BytesIO() as buffer:
+            dump_toml(config, buffer)
+
+            with open(self.pyproject_file, "wb+") as file_ptr:
+                file_ptr.write(buffer.getvalue())
+                logger.info(
+                    "Successfully saved configuration to %s",
+                    self.pyproject_file,
+                )
 
     @traced_function
     def _get_pyproject_config(self, section: _ConfigSection) -> _ConfigMapping:
-        project: _ModernPyProject = self.__project.pyproject
-        if isinstance(project, PyProject):
-            pyproject: PyProject = cast(PyProject, project)
-            return self._get_pyproject_file_value(pyproject, section)
+        project_data: _ConfigMapping = self._read_config()
+        section_key: Iterable[str] = ()
+        if section in (
+            _ConfigSection.TOOL_CONFIG,
+            _ConfigSection.PLUGIN_CONFIG,
+        ):
+            section_key = ["tool", "pdm"]
+            if _ConfigSection.PLUGIN_CONFIG == section:
+                section_key.extend(["plugins", "bump"])
+            section_key = tuple(section_key)
+        elif _ConfigSection.BUILD_SYSTEM == section:
+            section_key = ("build-system",)
+        elif _ConfigSection.METADATA == section:
+            section_key = ("project",)
+        elif _ConfigSection.ROOT == section:
+            return project_data or {}
 
-        if isinstance(project, dict):
-            data: _ConfigMapping = cast(_ConfigMapping, project)
-            return self._get_pyproject_raw_data_value(data, section)
+        data = _get_config_value(project_data, *section_key, default_value={})
 
-        logger.error("Failed to load project configuration.")
-        return {}
+        return cast(_ConfigMapping, data)
 
     @traced_function
-    def _get_pyproject_file_value(
-        self, pyproject: PyProject, section: _ConfigSection
-    ) -> _ConfigMapping:
-        if section == _ConfigSection.PLUGIN_CONFIG:
-            settings: Table = pyproject.settings
-            return cast(_ConfigMapping, settings.setdefault("plugins", {}))
-        if section == _ConfigSection.TOOL_CONFIG:
-            tool_settings: Table = pyproject.settings
-            return cast(_ConfigMapping, tool_settings)
-        if section == _ConfigSection.BUILD_SYSTEM:
-            build_system: dict = pyproject.build_system
-            return cast(_ConfigMapping, build_system)
-        if section == _ConfigSection.METADATA:
-            metadata: Table = pyproject.metadata
-            return cast(_ConfigMapping, metadata)
-
-        logger.error("Failed to determine section to load")
-        return {}
-
-    @traced_function
-    def _get_pyproject_raw_data_value(
-        self, data: _ConfigMapping, section: _ConfigSection
-    ) -> _ConfigMapping:
-        if section == _ConfigSection.PLUGIN_CONFIG:
-            for element in ["tools", "pdm", "plugins"]:
-                data = data[element] if element in data.keys() else {}
-            return cast(_ConfigMapping, data)
-        if section == _ConfigSection.TOOL_CONFIG:
-            for element in ["tools", "pdm"]:
-                data = data[element] if element in data.keys() else {}
-            return cast(_ConfigMapping, data)
-        if section == _ConfigSection.BUILD_SYSTEM:
-            return cast(
-                _ConfigMapping,
-                data["build-system"] if "build-system" in data.keys() else {},
-            )
-        if section == _ConfigSection.METADATA:
-            return cast(
-                _ConfigMapping,
-                data["project"] if "project" in data.keys() else {},
-            )
-
-        logger.error("Failed to determine section to load")
-        return {}
+    def _read_config(self) -> dict[str, Any]:
+        project_file = self.pyproject_file
+        with open(project_file, "rb") as file_pointer:
+            return load_toml(file_pointer)
